static char rcsid[] = "@(#)$Header: compfold.c,v 1.8 99/09/25 10:56:24 qhslyng Exp $";
/*
** compfold - compress mail folder
**
** (c) 1997     LYNCON
**
** History:
**
** 21-May-1997	1.1	Ted Lyngmo <ted@lyncon.se>
** Initial version
**
** 06-Jun-1997	1.2	Ted Lyngmo <ted@lyncon.se>
** Truncating Trash folder and its .snm-file.
** Skipping .dat-files.
**
** 06-Jun-1997	1.3	Ted Lyngmo <ted@lyncon.se>
** Added #include <unistd.h>
**
** 12-Jun-1997	1.4	Ted Lyngmo <ted@lyncon.se>
** All output to stdout (removed some stderr's)
**
** 28-Jul-1997	1.5	Ted Lyngmo <ted@lyncon.se>
** Truncating file after last email in case a deleted mail exceeded eof.
**
** 25-Sep-1999	1.8	Ted Lyngmo <ted@lyncon.se>
** Bug fix: Last mail in a mailbox was never deleted even though it was marked
** for deletion.
**
*/

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#define TRUE	(1)
#define FALSE	(0)

#define TRUNC	TRUE

t_strncpy( char *subject, char *inbuf, int maxlen )
{
    while( *inbuf == ' ' ) inbuf++;
    while( *inbuf >= ' ' && maxlen-- ) *subject++ = *inbuf++;
    *subject = 0;
}

#ifndef TRUNC
int
trunc( char *file )
{
    printf( "[truncating      ] \"%s\"\n", file );
    return( close( open( file, O_TRUNC ) ) );
}
#endif

int delmail( int mailstart, int delflag, int *deleted, FILE *fout, char *subject, int message ) {
    int
	changed=FALSE;

    if( mailstart != -1 && delflag == TRUE )
    {
	changed=TRUE;
	printf("[%-16s] (%d:%s)\n", "deleting mail", message, subject );
	*deleted=*deleted+1;
	*subject=0;
	fsetpos( fout, &mailstart );
    }
    return changed;
}

int
main( int argc, char *argv[] )
{
    int		no,
		error;
    FILE	*fin,
		*fout;
    char	inbuf[ BUFSIZ ],
		workfile[ BUFSIZ ],
		tmpfile[ BUFSIZ ],
		subject[ BUFSIZ ];

    umask( 0077 );

    sprintf( workfile, "cf%d.tmp",getpid() );

    for( no=1; no<argc; no++ )
    {
	if( !strcmp( argv[ no ], "Trash" ) )
	{
	    printf( "[truncating      ] \"%s\"\n", argv[ no ] );
#ifdef TRUNC
	    truncate( argv[ no ], 0);
#else
	    trunc( argv[ no ] );
#endif

	}
	else if( !strcmp( argv[ no ], "Trash.snm" ) )
	{
	    printf( "[truncating      ] \"%s\"\n", argv[ no ] );
#ifdef TRUNC
	    truncate( argv[ no ], 0);
#else
	    trunc( argv[ no ] );
#endif
	}
	else if( !strcmp( argv[ no ]+strlen( argv[ no ] )-4, ".dat" ) )
	{
	    printf( "[skipping datfile] \"%s\"\n", argv[ no ] );
	}
	else if( !strcmp( argv[ no ]+strlen( argv[ no ] )-4, ".snm" ) )
	{
	    printf( "[skipping index  ] \"%s\"\n", argv[ no ] );
	}
	else if( !(fin = fopen( argv[ no ], "r" ) ) )
	{
	    perror( argv[ no ] );
	}
	else
	{
	    int		delflag=FALSE,
			changed=FALSE,
			message=0,
			deleted=0;
	    fpos_t	foutpos=0,
			mailstart=-1;

	    printf("[%-16s] \"%s\"\n", "opened folder ok", argv[ no ] );

	    if( !(fout = fopen( workfile, "w+" ) ) )
	    {
		perror( workfile );
	    }
	    else
	    {
		error	= FALSE;

		while( error == FALSE && fgets( inbuf, BUFSIZ, fin ) != NULL )
		{
		    if( !strncmp( inbuf, "From ", 5 ) )
		    {
			if( delmail( mailstart, delflag, &deleted, fout, subject, message ) )
			    changed=TRUE;

			message++;
			delflag = FALSE;
			mailstart = ftell( fout );
		    }
		    else if( !strncmp( inbuf, "X-Mozilla-Status: ", 18 ) )
		    {
			if( inbuf[21] == '9' )
			{
			    delflag = TRUE;
			    printf("[%-16s] marked mail %d for deletion\n", "found old mail", message );
			}
		    }
		    else if( !strncmp( inbuf, "Subject:", 8 ) )
		    {
			t_strncpy( subject, inbuf+8, BUFSIZ );
		    }

		    if( fputs( inbuf, fout ) == EOF )
		    {
			printf("[%-16s] %s (%d)\n", "ERROR", strerror(errno), errno );
			error	= TRUE;
			changed	= 0;
		    }

		    foutpos = ftell( fout );
		}
		delmail( mailstart, delflag, &deleted, fout, subject, message );
#ifdef TRUNC
		foutpos = ftell( fout );
		ftruncate( fileno( fout ), foutpos );
#endif
		fclose( fout );
	    }
	    fclose( fin );

	    if( fout )
	    {
		if( changed )
		{
		    rename( workfile, argv[ no ] );

		    sprintf( tmpfile, "%s.snm", argv[ no ] );
		    remove( tmpfile );

		    sprintf( tmpfile, ".%s.summary", argv[ no ] );
		    remove( tmpfile );
		    printf("[%-16s] deleted %d of %d, keeping %d messages\n", "folder changed", deleted, message, message-deleted );
		}
		else
		{
		    remove( workfile );
		    printf("[%-16s] keeping %d messages\n", "folder unchanged", message-deleted );
		}
	    }
	}
    }
}
